<?php
/**
 * Copyright 2010, 2011 pixeltricks GmbH
 *
 * This file is part of SilvercartPdfInvoice.
 *
 * SilvercartPdfInvoice is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilvercartPdfInvoice is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilvercartPdfInvoice.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package Silvercart
 * @subpackage PdfInvoice
 */

/**
 * PDF invoice module
 *
 * @package Silvercart
 * @subpackage PdfInvoice
 * @author Sebastian Diel <sdiel@pixeltricks.de>
 * @copyright 2011 pixeltricks GmbH
 * @since 25.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartPdfInvoice extends Zend_Pdf {
    
    /**
     * SilvercartPdfInvoiceConfig object
     *
     * @var SilvercartPdfInvoiceConfig
     */
    protected $config = null;

    /**
     *
     * @var SilvercartOrder
     */
    protected $order;

    // PDF specific data
    
    /**
     * Encoding of the PDF
     *
     * @var string
     */
    protected $encoding;
    
    /**
     * Filename to use
     *
     * @var string
     */
    protected $filename;

    /**
     * Name of default font to use.
     *
     * @var string 
     */
    protected $fontName = null;
    
    /**
     * Name of bold font to use.
     *
     * @var string 
     */
    protected $fontBoldName = null;
    
    /**
     * Name of bold italic font to use.
     *
     * @var string 
     */
    protected $fontBoldItalicName = null;
    
    /**
     * Name of bold oblique font to use.
     *
     * @var string 
     */
    protected $fontBoldObliqueName = null;
    
    /**
     * Name of italic font to use.
     *
     * @var string 
     */
    protected $fontItalicName = null;
    
    /**
     * Name of oblique font to use.
     *
     * @var string 
     */
    protected $fontObliqueName = null;
    
    /**
     * Default font to use.
     *
     * @var Zend_Pdf_Resource_Font 
     */
    protected $font = null;
    
    /**
     * Bold font to use.
     *
     * @var Zend_Pdf_Resource_Font 
     */
    protected $fontBold = null;
    
    /**
     * Bold italic font to use.
     *
     * @var Zend_Pdf_Resource_Font 
     */
    protected $fontBoldItalic = null;
    
    /**
     * Name of bold oblique font to use.
     *
     * @var Zend_Pdf_Resource_Font 
     */
    protected $fontBoldOblique = null;
    
    /**
     * Name of italic font to use.
     *
     * @var Zend_Pdf_Resource_Font 
     */
    protected $fontItalic = null;
    
    /**
     * Name of oblique font to use.
     *
     * @var Zend_Pdf_Resource_Font 
     */
    protected $fontOblique = null;

    /**
     * Left border position to draw anything
     *
     * @var int
     */
    protected $left = 0;

    /**
     * The locale to restore after generating the PDF
     *
     * @var type string
     */
    protected $oldLocale = null;
    
    /**
     * Current PDF page
     *
     * @var Zend_Pdf_Page
     */
    protected $page = null;

    /**
     * Size of page
     *
     * @var string 
     */
    protected $pageSize = null;

    /**
     * Right border position to draw anything
     *
     * @var int
     */
    protected $right = 0;
    
    /**
     * Current y position of the page
     *
     * @var int
     */
    protected $yPosition = 0;

    /**
     * Default constructor.
     * Sets default properties and the system locale.
     *
     * @param SilvercartOrder $order    SilvercartOrder to write
     * @param string          $source   PDF file to load
     * @param int             $revision Revision of PDF file
     * @param bool            $load     Load file or not
     * 
     * @return Zend_Pdf
     * 
     * @throws Zend_Pdf_Exception
     */
    public function __construct(SilvercartOrder $order = null, $source = null, $revision = null, $load = false) {
        $this->switchLocale();
        parent::__construct($source, $revision, $load);
        // initialize default properties
        $this->properties['Encoding']      = 'UTF-8';
        $this->properties['Title']         = 'Rechnung';
        $this->properties['Author']        = 'SilverCart';
        $this->properties['Creator']       = 'http://www.silvercart.org/';
        // set default encoding
        $this->setEncoding('UTF-8');
        // set default page size
        $this->setPageSize(Zend_Pdf_Page::SIZE_A4);
        // set default font names
        $this->setFontName(Zend_Pdf_Font::FONT_HELVETICA);
        $this->setFontBoldName(Zend_Pdf_Font::FONT_HELVETICA_BOLD);
        $this->setFontBoldItalicName(Zend_Pdf_Font::FONT_HELVETICA_BOLD_ITALIC);
        $this->setFontBoldObliqueName(Zend_Pdf_Font::FONT_HELVETICA_BOLD_OBLIQUE);
        $this->setFontItalicName(Zend_Pdf_Font::FONT_HELVETICA_ITALIC);
        $this->setFontObliqueName(Zend_Pdf_Font::FONT_HELVETICA_OBLIQUE);
        // set default fonts
        $this->setFont(Zend_Pdf_Font::fontWithName($this->getFontName()));
        $this->setFontBold(Zend_Pdf_Font::fontWithName($this->getFontBoldName()));
        $this->setFontBoldItalic(Zend_Pdf_Font::fontWithName($this->getFontBoldItalicName()));
        $this->setFontBoldOblique(Zend_Pdf_Font::fontWithName($this->getFontBoldObliqueName()));
        $this->setFontItalic(Zend_Pdf_Font::fontWithName($this->getFontItalicName()));
        $this->setFontOblique(Zend_Pdf_Font::fontWithName($this->getFontObliqueName()));
        // positions/coords
        $this->setLeft(40);
        $this->setRight(560);
        // set configuration
        $this->setConfig(SilvercartPdfInvoiceConfig::getConfig());
        
        if (!is_null($order)) {
            $this->setOrder($order);
            $this->setFilename($this->getOrder()->ID . '-' . $this->getOrder()->InvoiceNumber . '.pdf');
        }
    }
    
    /**
     * Default destructor.
     * Resets the system locale.
     */
    public function __destruct() {
        $this->switchLocale();
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // GETTER SECTION
    ////////////////////////////////////////////////////////////////////////////
    
    /**
     * Returns the SilvercartPdfInvoiceConfig object
     *
     * @return SilvercartPdfInvoiceConfig
     */
    public function getConfig() {
        return $this->config;
    }

    /**
     * Returns the encoding of the PDF
     *
     * @return string
     */
    public function getEncoding() {
        return $this->encoding;
    }

    /**
     * Returns the filename to use
     *
     * @return string
     */
    public function getFilename() {
        return $this->filename;
    }

    /**
     * Returns the default font
     *
     * @return Zend_Pdf_Resource_Font
     */
    public function getFont() {
        return $this->font;
    }

    /**
     * Returns the default font name
     *
     * @return string
     */
    public function getFontName() {
        return $this->fontName;
    }

    /**
     * Returns the bold font
     *
     * @return Zend_Pdf_Resource_Font
     */
    public function getFontBold() {
        return $this->fontBold;
    }

    /**
     * Returns the bold font name
     *
     * @return string
     */
    public function getFontBoldName() {
        return $this->fontBoldName;
    }

    /**
     * Returns the bold italic font
     *
     * @return Zend_Pdf_Resource_Font
     */
    public function getFontBoldItalic() {
        return $this->fontBoldItalic;
    }

    /**
     * Returns the bold italic font name
     *
     * @return string
     */
    public function getFontBoldItalicName() {
        return $this->fontBoldItalicName;
    }

    /**
     * Returns the bold oblique font
     *
     * @return Zend_Pdf_Resource_Font
     */
    public function getFontBoldOblique() {
        return $this->fontBoldOblique;
    }

    /**
     * Returns the bold oblique font name
     *
     * @return string
     */
    public function getFontBoldObliqueName() {
        return $this->fontBoldObliqueName;
    }

    /**
     * Returns the italic font
     *
     * @return Zend_Pdf_Resource_Font
     */
    public function getFontItalic() {
        return $this->fontItalic;
    }

    /**
     * Returns the italic font name
     *
     * @return string
     */
    public function getFontItalicName() {
        return $this->fontItalicName;
    }

    /**
     * Returns the oblique font
     *
     * @return Zend_Pdf_Resource_Font
     */
    public function getFontOblique() {
        return $this->fontOblique;
    }

    /**
     * Returns the oblique font name
     *
     * @return string
     */
    public function getFontObliqueName() {
        return $this->fontObliqueName;
    }
    
    /**
     * Returns the left border position
     *
     * @return int 
     */
    public function getLeft() {
        return $this->left;
    }

    /**
     * Get the related SilvercartOrder.
     *
     * @return SilvercartOrder
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    public function getOrder() {
        return $this->order;
    }
    
    /**
     * Returns the current PDF page
     *
     * @return Zend_Pdf_Page
     */
    public function getPage() {
        return $this->page;
    }

    /**
     * Returns the page size
     *
     * @return string
     */
    public function getPageSize() {
        return $this->pageSize;
    }
    
    /**
     * Returns the right border position
     *
     * @return int 
     */
    public function getRight() {
        return $this->right;
    }
    
    /**
     * Returns the current y position
     *
     * @return int
     */
    public function getYPosition() {
        return $this->yPosition;
    }
    
    /**
     * Returns and reduces the current y position
     *
     * @param int $reduceAmount reducement
     * 
     * @return int
     */
    public function getAndReduceYPosition($reduceAmount) {
        $y = $this->yPosition;
        $this->yPosition -= $reduceAmount;
        return $y;
    }


    ////////////////////////////////////////////////////////////////////////////
    // SETTER SECTION
    ////////////////////////////////////////////////////////////////////////////
    
    /**
     * Sets the SilvercartPdfInvoiceConfig object
     *
     * @param SilvercartPdfInvoiceConfig $config SilvercartPdfInvoiceConfig object
     * 
     * @return void
     */
    public function setConfig(SilvercartPdfInvoiceConfig $config) {
        $this->config = $config;
    }

    /**
     * Sets the encoding of the PDF
     *
     * @param type $encoding Encoding of the PDF
     * 
     * @return void
     */
    public function setEncoding($encoding) {
        $this->encoding = $encoding;
    }

    /**
     * Sets the filename to use
     *
     * @param string $filename filename to use
     * 
     * @return void
     */
    public function setFilename($filename) {
        $this->filename = $filename;
    }

    /**
     * Sets the default font
     *
     * @param Zend_Pdf_Resource_Font $font default font
     * 
     * @return void
     */
    public function setFont($font) {
        $this->font = $font;
    }

    /**
     * Sets the default font name
     *
     * @param string $fontName default font name
     * 
     * @return void
     */
    public function setFontName($fontName) {
        $this->fontName = $fontName;
    }

    /**
     * Sets the bold font
     *
     * @param Zend_Pdf_Resource_Font $fontBold bold font
     * 
     * @return void
     */
    public function setFontBold($fontBold) {
        $this->fontBold = $fontBold;
    }

    /**
     * Sets the bold font name
     *
     * @param string $fontBoldName bold font name
     * 
     * @return void
     */
    public function setFontBoldName($fontBoldName) {
        $this->fontBoldName = $fontBoldName;
    }

    /**
     * Sets the bold italic font
     *
     * @param Zend_Pdf_Resource_Font $fontBoldItalic bold italic font
     * 
     * @return void
     */
    public function setFontBoldItalic($fontBoldItalic) {
        $this->fontBoldItalic = $fontBoldItalic;
    }

    /**
     * Sets the bold italic font name
     *
     * @param string $fontBoldItalicName bold italic font name
     * 
     * @return void
     */
    public function setFontBoldItalicName($fontBoldItalicName) {
        $this->fontBoldItalicName = $fontBoldItalicName;
    }

    /**
     * Sets the bold oblique font
     *
     * @param Zend_Pdf_Resource_Font $fontBoldOblique bold oblique font
     * 
     * @return void
     */
    public function setFontBoldOblique($fontBoldOblique) {
        $this->fontBoldOblique = $fontBoldOblique;
    }

    /**
     * Sets the bold oblique font name
     *
     * @param string $fontBoldObliqueName bold oblique font name
     * 
     * @return void
     */
    public function setFontBoldObliqueName($fontBoldObliqueName) {
        $this->fontBoldObliqueName = $fontBoldObliqueName;
    }

    /**
     * Sets the italic font
     *
     * @param Zend_Pdf_Resource_Font $fontItalic italic font
     * 
     * @return void
     */
    public function setFontItalic($fontItalic) {
        $this->fontItalic = $fontItalic;
    }

    /**
     * Sets the italic font name
     *
     * @param string $fontItalicName italic font name
     * 
     * @return void
     */
    public function setFontItalicName($fontItalicName) {
        $this->fontItalicName = $fontItalicName;
    }

    /**
     * Sets the oblique font
     *
     * @param Zend_Pdf_Resource_Font $fontOblique oblique font
     * 
     * @return void
     */
    public function setFontOblique($fontOblique) {
        $this->fontOblique = $fontOblique;
    }

    /**
     * Sets the oblique font name
     *
     * @param string $fontObliqueName oblique font name
     * 
     * @return void
     */
    public function setFontObliqueName($fontObliqueName) {
        $this->fontObliqueName = $fontObliqueName;
    }
    
    /**
     * Sets the left border position
     * 
     * @param int $left Left border position
     *
     * @return void 
     */
    public function setLeft($left) {
        $this->left = $left;
    }

    /**
     * Set the related SilvercartOrder.
     *
     * @param SilvercartOrder $order related SilvercartOrder
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    public function setOrder(SilvercartOrder $order) {
        $this->order = $order;
    }
    
    /**
     * Sets the current PDF page
     *
     * @param Zend_Pdf_Page $page Page
     * 
     * @return void
     */
    public function setPage(Zend_Pdf_Page $page) {
        $this->page = $page;
    }

    /**
     * Sets the page size
     *
     * @param string $pageSize The page size to use
     * 
     * @return void
     */
    public function setPageSize($pageSize) {
        $this->pageSize = $pageSize;
    }
    
    /**
     * Sets the right border position
     * 
     * @param int $right Right border position
     *
     * @return void 
     */
    public function setRight($right) {
        $this->right = $right;
    }
    
    /**
     * Sets the current y position
     *
     * @param int $yPosition new y position
     * 
     * @return void
     */
    public function setYPosition($yPosition) {
        $this->yPosition = $yPosition;
    }
    
    /**
     * Reduces the current y position with $reduceAmount
     *
     * @param int $reduceAmount amount to reduce the y position with
     * 
     * @return void
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    public function reduceYPosition($reduceAmount) {
        $this->yPosition -= $reduceAmount;
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // EXECUTIVE SECTION
    ////////////////////////////////////////////////////////////////////////////

    /**
     * Generates and saves the PDF.
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.05.2011
     */
    public function generateInvoice() {
        $page = $this->newPage($this->getPageSize());
        $this->setPage($page);

        $this->drawHeader();
        $this->drawLetterHead();
        $this->drawMetaData();
        $this->drawInvoiceHead();
        $this->drawPositions();
        $this->drawOrderSum();
        $this->drawFooter();
        
        $this->pages[] = $this->getPage();
        
        $basePath = $this->getConfig()->getBasePath();
        $relativePath = str_replace(Director::baseFolder() . '/', '', $basePath);
        if (!is_dir($basePath)) {
            $this->createDirectory($basePath);
        }
        $filename = SilvercartPdfInvoiceConfig::$rewriteRuleDir . '/';
        $parts = explode('/', SilvercartPdfInvoiceConfig::$rewriteRuleDir);
        $name = $parts[count($parts) - 1];
        $dir = DataObject::get_one('File', sprintf("`Filename` = '%s' AND `Name` = '%s'", $filename, $name));
        if (!$dir) {
            $dir = new Folder();
            $dir->Filename = $filename;
            $dir->Name = $name;
            $dir->Title = $name;
            $dir->write();
        }
        
        $realFilename = $basePath . $this->getFilename();
        
        if ($this->getOrder()->SilvercartPdfInvoiceFileID &&
            $this->getOrder()->SilvercartPdfInvoiceFile()) {
            $pdf = $this->getOrder()->SilvercartPdfInvoiceFile();
        } else {
            $pdf = new SilvercartPdfInvoiceFile();
            $fileInfo = pathinfo($realFilename);
            $pdf->Filename = $dir->Filename . $this->getFilename();
            $pdf->Title = empty($fileInfo['extension']) ? $this->getFilename() : basename($realFilename, '.' . $fileInfo['extension']);
            $pdf->ParentID = $dir->ID;
            $pdf->write();
            $this->getOrder()->SilvercartPdfInvoiceFileID = $pdf->ID;
            $this->switchLocale();
            $this->getOrder()->write();
            $this->switchLocale();
        }

        $this->save($realFilename);
        
        return $realFilename;
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // HELPER SECTION
    ////////////////////////////////////////////////////////////////////////////
    
    /**
     * Creates a directory
     *
     * @param string $path Path to create
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 26.08.2011
     */
    protected function createDirectory($path) {
        $parts = explode('/', $path);
        $path = '';
        foreach ($parts as $part) {
            $path .= '/' . $part;
            if (!is_dir($path)) {
                mkdir($path);
            }
        }
    }


    /**
     * Draws a horizontal line
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    protected function drawLine() {
        $x1 = $this->getLeft();
        $y1 = $this->getYPosition();
        $x2 = $this->getRight();
        $y2 = $this->getAndReduceYPosition(15);
        $this->getPage()->drawLine($x1, $y1, $x2, $y2);
    }
    
    /**
     * Draws a multiline text
     *
     * @param string                 $text     Text
     * @param int                    $x        y position
     * @param int                    $y        y position
     * @param int                    $length   Length of line
     * @param string                 $align    Alignment
     * @param Zend_Pdf_Resource_Font $font     Font
     * @param int                    $fontSize Font size
     * 
     * @return int
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    protected function drawMultiLineText($text, $x, $y, $length, $align = 'left', $font = null, $fontSize = null) {
        $multiLineText = wordwrap($text, $length, "\n", true);
        $lines = explode("\n", $multiLineText);
        
        if (is_null($font)) {
            $fontSize = $this->getPage()->getFontSize();
        } else {
            $this->getPage()->setFont($font, $fontSize);
        }

        $maxLines = 4;
        $lineIndex = 0;
        foreach ($lines as $line) {
            if ($lineIndex == $maxLines) {
                break;
            }
            $this->drawText($line, $x, $y, $align);
            $y = $y - ($fontSize + 2);
            $lineIndex++;
        }
        
        return $y;
    }
    
    /**
     * Draws the given text to the given coords of the PDF page.
     *
     * @param string $text  Text to draw
     * @param int    $x     x position
     * @param int    $y     y position
     * @param string $align Alignment (left/right) left is default
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 26.08.2011
     */
    protected function drawText($text, $x, $y, $align = 'left') {
        if ($align == 'right') {
            $x  = $x - $this->fontStringWidth($text, $this->getPage()->getFont(), $this->getPage()->getFontSize());
        }
        $this->getPage()->drawText($text, $x, $y, $this->getEncoding());
    }

    /**
     * Calculates the width of a string.
     *
     * @param string                 $string   String to check
     * @param Zend_Pdf_Resource_Font $font     Font to use
     * @param int                    $fontSize Font size to use
     * 
     * @return int
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.05.2011
     */
    protected function fontStringWidth($string, Zend_Pdf_Resource_Font $font, $fontSize) {
        $patternS = array(('ö'),('ü'),('ä'),('Ö'),('Ü'),('Ä'),('ß'));
        $replaceS = array(('o'),('u'),('a'),('O'),('U'),('A'),('S'));
        $string = str_replace($patternS,$replaceS,$string);
        $drawingString = iconv('', 'UTF-16BE', $string);
        $characters = array();
        for ($i = 0; $i < strlen($drawingString); $i++) {
            $characters[] = (ord($drawingString[$i++]) << 8) | ord($drawingString[$i]);
        }
        $glyphs = $font->glyphNumbersForCharacters($characters);
        $widths = $font->widthsForGlyphs($glyphs);
        $stringWidth = (array_sum($widths) / $font->getUnitsPerEm()) * $fontSize;
        return $stringWidth;
    }
    
    /**
     * Calculates the start position of a string to get a centered alignment.
     *
     * @param string                 $string   String to check
     * @param Zend_Pdf_Resource_Font $font     Font to use
     * @param int                    $fontSize Font size to use
     * 
     * @return int
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.05.2011
     */
    protected function getCenteredPosition($string, Zend_Pdf_Resource_Font $font, $fontSize) {
        list($width, $height) = explode(':', $this->getPageSize());
        $center = (int) $width / 2;
        $centeredPosition = $center - ($this->fontStringWidth($string, $font, $fontSize) / 2);
        return $centeredPosition;
    }
    
    /**
     * Switchs the locale from default to the current SS locale and back.
     * This method is called in constructor and destructor.
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 18.05.2011
     */
    protected function switchLocale() {
        if (is_null($this->oldLocale)) {
            $this->oldLocale = setlocale(LC_ALL, null);
            $currentLocale = Translatable::get_current_locale();
        } else {
            $currentLocale = $this->oldLocale;
            $this->oldLocale = null;
        }
        // it's a kind of dirty, because this will not match every possible
        // system locale... It works for plain and utf8 locales.
        $result = setlocale(LC_ALL, $currentLocale . '.utf8', $currentLocale . '.UTF-8', $currentLocale);
    }
    
    /**
     * Returns the scaled image dimensions.
     *
     * @param Zend_Pdf_Resource_Image $image     Image
     * @param int                     $maxWidth  Maximum width
     * @param int                     $maxHeight Maximum height
     * 
     * @return array 
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    protected function getScaledDimensions(Zend_Pdf_Resource_Image $image, $maxWidth, $maxHeight) {
        $imageHeight = $image->getPixelHeight();
        $imageWidth = $image->getPixelWidth();
        
        $widthRatio = $maxWidth / $imageWidth;
        $heightRatio = $maxHeight / $imageHeight;
        
        if ( $widthRatio < $heightRatio ) {
            $heightScale = $maxWidth / $imageWidth;
            $imageWidth = $maxWidth;
            $imageHeight = $heightScale * $imageHeight;
        } else {
            $widthScale = $maxHeight / $imageHeight;
            $imageHeight = $maxHeight;
            $imageWidth = $widthScale * $imageWidth;
        }
        
        return array(
            $imageWidth,
            $imageHeight,
        );
    }


    ////////////////////////////////////////////////////////////////////////////
    // LAYOUT SECTION
    ////////////////////////////////////////////////////////////////////////////
    
    /**
     * Draws the header of the PDF
     * 
     * @return int 
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.05.2011
     */
    protected function drawHeader() {
        if ($this->getConfig()->LayoutLogoID) {
            $this->setYPosition(731);
            $imageFile = $this->getConfig()->LayoutLogo()->getFullPath();
            $stampImage = Zend_Pdf_Image::imageWithPath($imageFile);

            list(
                $imageWidth,
                $imageHeight
            ) = $this->getScaledDimensions($stampImage, 400, 200);

            if ($this->getConfig()->LayoutLogoPosition == "Left") {
                $x1 = 37;
                $y1 = $this->getYPosition();
                $x2 = $x1 + $imageWidth/2;
                $y2 = $y1 + $imageHeight/2;
            } else {
                $x1 = 405;
                $y1 = $this->getYPosition();
                $x2 = $x1 + $imageWidth/2;
                $y2 = $y1 + $imageHeight/2;
            }
            

            $this->getPage()->drawImage($stampImage, $x1, $y1, $x2, $y2);
        }
    }
    
    /**
     * Draws the letter head
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    protected function drawLetterHead() {
        $this->setYPosition(690);
        $invoiceAddress = $this->getOrder()->SilvercartInvoiceAddress();
        
        $defaultFontSize = 9;
        $lines = array(
            array(
                'fontsize'  => 8,
                'value'     => $this->getConfig()->LayoutSenderAddress,
            ),
            '',
            '',
            $invoiceAddress->SalutationText . ' ' . $invoiceAddress->FirstName . ' ' . $invoiceAddress->Surname,
            $invoiceAddress->Street . ' ' . $invoiceAddress->StreetNumber,
            $invoiceAddress->Postcode . ' ' . $invoiceAddress->City,
            '',
            $invoiceAddress->SilvercartCountry()->Title,
        );
        
        foreach ($lines as $line) {
            if (is_array($line)) {
                $value = $line['value'];
                $fontsize = $line['fontsize'];
            } else {
                $value = $line;
                $fontsize = $defaultFontSize;
            }
            $this->getPage()->setFont($this->getFont(), $fontsize);
            $this->drawText($value, $this->getLeft(), $this->getAndReduceYPosition(10));
        }
    }
    
    /**
     * Draws the meta data
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    protected function drawMetaData() {
        $this->setYPosition(690);
        $lines = array(
            _t('SilvercartCustomer.CUSTOMERNUMBER_SHORT')       => $this->getOrder()->Member()->CustomerNumber,
            _t('SilvercartPdfInvoiceConfig.INVOICENUMBER_SHORT')    => $this->getOrder()->InvoiceNumber,
        );
        
        $this->getPage()->setFont($this->getFont(), 9);
        foreach ($lines as $label => $value) {
            $label .= ':';
            $this->drawText($label, 405, $this->getYPosition());
            $this->drawText($value, $this->getRight(), $this->getAndReduceYPosition(10), 'right');
        }
    }
    
    /**
     * Draws the invoice head
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    protected function drawInvoiceHead() {
        $this->setYPosition(540);
        
        $this->getPage()->setFont($this->getFontBold(), 10);
        $this->drawText(_t('SilvercartPdfInvoice.INVOICE'), $this->getLeft(), $this->getYPosition());
        $this->getPage()->setFont($this->getFont(), 9);
        $this->drawText(strftime('%d. %B %Y'), $this->getRight(), $this->getAndReduceYPosition(30), 'right');
        
        $this->getPage()->setFont($this->getFontBold(), 9);
        $tableHeadData = array(
            _t('SilvercartOrderPosition.SHORT') => 40,
            _t('SilvercartProduct.QUANTITY') => 40,
            _t('SilvercartProduct.PRODUCTNUMBER_SHORT') => 60,
            _t('SilvercartProduct.TITLE') => 260,
            _t('SilvercartProduct.PRICE_SINGLE') => 60,
            _t('SilvercartProduct.PRICE_ENTIRE') => 60,
        );
        
        $x = $this->getLeft();
        foreach ($tableHeadData as $label => $length) {
            $this->drawText($label, $x, $this->getYPosition());
            $x += $length;
        }
        $this->reduceYPosition(10);
        $this->drawLine();
    }
    
    /**
     * Draws the positions
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    protected function drawPositions() {
        $orderPositions = $this->getOrder()->SilvercartOrderPositions();
        $this->getPage()->setFont($this->getFont(), 9);
        $fields = array(
            'PdfPosCount'           => 40,
            'Quantity'              => 40,
            'ProductNumber'         => 60,
            'Title'                 => 260,
            'PriceNice'             => array(
                'length' => 60,
                'align'  => 'right',
            ),
            'PriceTotalNice'        => array(
                'length' => 60,
                'align'  => 'right',
            ),
        );
        
        $pos = 1;
        $tmpPos = $this->getYPosition();
        foreach ($orderPositions as $orderPosition) {
            $x = $this->getLeft();
            $mlPos = 0;
            $orderPosition->PdfPosCount = $pos;
            foreach ($fields as $value => $length) {
                $tmpX = 0;
                if (is_array($length)) {
                    $align = $length['align'];
                    $length = $length['length'];
                } else {
                    $align = 'left';
                }
                if ($align == 'right') {
                    $x += $length;
                }
                $mlPos = $this->drawMultiLineText(strip_tags($orderPosition->$value), $x, $this->getYPosition(), $length/4, $align);
                if ($mlPos < $tmpPos) {
                    $tmpPos = $mlPos;
                }
                if ($align == 'left') {
                    $x += $length;
                }
            }
            $pos++;
            $this->setYPosition($tmpPos);
            $this->drawLine();
        }
        
    }
    
    /**
     * Draws the order sum
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    protected function drawOrderSum() {
        $taxAmount      = 0;
        $taxCurrency    = '';
        foreach ($this->getOrder()->getTaxRatesWithFees() as $taxRate) {
            $taxAmount += $taxRate->Amount->getAmount();
            $taxCurrency = $taxRate->Amount->getCurrency();
        }
        
        $paymentLabel   = _t('SilvercartPaymentMethod.SINGULARNAME') . ' (' . $this->getOrder()->SilvercartPaymentMethod()->Name . ')';
        $shipmentLabel  = _t('SilvercartShippingMethod.SINGULARNAME') . ' (' . $this->getOrder()->SilvercartShippingMethod()->Title . ')';
        $amountGrossTotalLabel = _t('SilvercartOrder.AMOUNTGROSSTOTAL') . ' (' . _t('SilvercartPage.INCLUDED_VAT') . ': ' . str_replace('.', ',', number_format($taxAmount, 2)) . ' ' . $taxCurrency . ')';
        
        $lines = array(
            _t('SilvercartPage.SUBTOTAL')           => str_replace('.', ',', number_format($this->getOrder()->getTaxableAmountGross()->getAmount(), 2)) . ' ' . $this->getOrder()->getTaxableAmountGross()->getCurrency(),
            $paymentLabel                           => str_replace('.', ',', number_format($this->getOrder()->HandlingCostPayment->getAmount(), 2)) . ' ' . $this->getOrder()->HandlingCostPayment->getCurrency(),
            $shipmentLabel                          => str_replace('.', ',', number_format($this->getOrder()->HandlingCostShipment->getAmount(), 2)) . ' ' . $this->getOrder()->HandlingCostShipment->getCurrency(),
            'line',
            $amountGrossTotalLabel                  => array(
                'value' => str_replace('.', ',', number_format($this->getOrder()->AmountTotal->getAmount(), 2)) . ' ' . $this->getOrder()->AmountTotal->getCurrency(),
                'font' => 'getFontBold'
            ),
        );
        
        foreach ($lines as $label => $value) {
            if ($value == 'line') {
                $this->drawLine();
            } else {
                if (is_array($value)) {
                    $font = $value['font'];
                    $value = $value['value'];
                } else {
                    $font = 'getFont';
                }
                $this->getPage()->setFont($this->$font(), 9);
                $label .= ':';
                $this->drawText($label, 450, $this->getYPosition(), 'right');
                $this->drawText($value, $this->getRight(), $this->getAndReduceYPosition(10), 'right');
            }
        }
        $this->drawLine();
    }
    
    /**
     * Draws the footer of the PDF
     * 
     * @return int 
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.05.2011
     */
    protected function drawFooter() {
        $this->setYPosition(67);
        $this->drawLine();
        
        $this->setYPosition(150);
        $hint = 'Herzlichen Dank!';
        $hint2 = 'Bitte bedenken Sie, dass Sie als Privatperson diese Rechnung 2 Jahre aufbewahren müssen. Evtl. Verpackungsschäden bitte sofort beim anliefernden Paketdienst reklamieren (quittieren lassen).';
        $hint3 = 'Leistungszeitpunkt und Rechnungsdatum stimmen überein.';
        
        $this->getPage()->setFont($this->getFont(), 9);
        $this->drawText($hint, $this->getLeft(), $this->getAndReduceYPosition(40));
        $this->getPage()->setFont($this->getFontBold(), 9);
        $newPos = $this->drawMultiLineText($hint2, $this->getLeft(), $this->getYPosition(), 100);
        $this->setYPosition($newPos);
        $this->getPage()->setFont($this->getFont(), 9);
        $this->drawText($hint3, $this->getLeft(), $this->getYPosition());
        
        $col1 = explode("\n", $this->getConfig()->LayoutFooterColumn1);
        $col2 = explode("\n", $this->getConfig()->LayoutFooterColumn2);
        $col3 = explode("\n", $this->getConfig()->LayoutFooterColumn3);
        
        if ($this->getConfig()->LayoutFooter == "1") {
            if ($this->getConfig()->LayoutLogoID) {
                $imageFile = $this->getConfig()->LayoutLogo()->getFullPath();
                $stampImage = Zend_Pdf_Image::imageWithPath($imageFile);

                list(
                    $imageWidth,
                    $imageHeight
                ) = $this->getScaledDimensions($stampImage, 200, 100);

                $this->setYPosition(25);   
                $x1 = $this->getLeft();
                $y1 = $this->getYPosition();
                $x2 = $x1 + $imageWidth/2;
                $y2 = $y1 + $imageHeight/2;

                $this->getPage()->drawImage($stampImage, $x1, $y1, $x2, $y2);
            }
            $cols = array(
                140 => $col1,
                275 => $col2,
                455 => $col3,
            );
        } else {
            $col4 = explode("\n", $this->getConfig()->LayoutFooterColumn4);
            $cols = array(
                $this->getLeft()    => $col1,
                170                 => $col2,
                300                 => $col3,
                430                 => $col4,
            );
        }
        
        $this->getPage()->setFont($this->getFont(), 8);
        foreach ($cols as $xPos => $lines) {
            $this->setYPosition(55);
            foreach ($lines as $line) {
                $this->drawText($line, $xPos, $this->getAndReduceYPosition(10));
            }
        }
    }
    
}