<?php
/**
 * Copyright 2010, 2011 pixeltricks GmbH
 *
 * This file is part of SilvercartPdfInvoice.
 *
 * SilvercartPdfInvoice is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilvercartPdfInvoice is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilvercartPdfInvoice.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package Silvercart
 * @subpackage PdfInvoice
 */

/**
 * Controller to display PDF invoice files
 * 
 * @package Silvercart
 * @subpackage PdfInvoice
 * @author Sebastian Diel <sdiel@pixeltricks.de>
 * @copyright 2011 pixeltricks GmbH
 * @since 26.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercarPdfInvoice_Controller extends Controller {
    
    /**
     * Checks the given data to display the invoice.
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 26.08.2011
     */
    public function init() {
        parent::init();
        if (Member::currentUserID()) {
            $allParams = $this->request->allParams();
            $orderID = false;
            if (is_array($allParams) &&
                array_key_exists('ID', $allParams) &&
                !empty ($allParams['ID']) &&
                array_key_exists('OtherID', $allParams) &&
                !empty ($allParams['OtherID'])) {
                $orderID = $allParams['ID'];
                $filename = $allParams['OtherID'];
                if (strpos($filename, '.pdf') === false) {
                    $filename .= '.pdf';
                }
            } elseif (is_array($allParams) &&
                array_key_exists('ID', $allParams) &&
                !empty ($allParams['ID'])) {
                $filename = $allParams['ID'];
                if (strpos($filename, '.pdf') === false) {
                    $filename .= '.pdf';
                }
                $parts = explode('-', $filename);
                $orderID = $parts[0];
            }
            if ($orderID) {
                if (Member::currentUser()->isAdmin()) {
                    $order = DataObject::get_by_id('SilvercartOrder', $orderID);
                } else {
                    $order = DataObject::get_one('SilvercartOrder', sprintf("`ID` = '%s' AND `MemberID` = '%s'", $orderID, Member::currentUserID()));
                }

                if ($order) {
                    $invoice = $order->SilvercartPdfInvoiceFile();
                    if ($invoice) {
                        $config = SilvercartPdfInvoiceConfig::getConfig();
                        $pdfFilePath = $config->getBasePath() . $filename;
                        if (file_exists($pdfFilePath)) {
                            header('Content-Type: application/pdf');
                            print file_get_contents($config->getBasePath() . $filename);
                            exit();
                        }
                        // invoice file does not exist
                    }
                    // no invoice found
                }
                // no order found with this id and customer
            }
            // no params given
        }
        // no member logged in
        Director::redirect('/');
    }
    
}