<?php
/**
 * Copyright 2010, 2011 pixeltricks GmbH
 *
 * This file is part of SilvercartPdfInvoice.
 *
 * SilvercartPdfInvoice is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilvercartPdfInvoice is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilvercartPdfInvoice.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package Silvercart
 * @subpackage PdfInvoice
 */

/**
 * PDF invoice module
 *
 * @package Silvercart
 * @subpackage PdfInvoice
 * @author Sebastian Diel <sdiel@pixeltricks.de>
 * @copyright 2011 pixeltricks GmbH
 * @since 25.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartPdfInvoiceOrderDecorator extends DataObjectDecorator {
    
    protected $pdfInvoiceFilePath = null;


    /**
     * Adds a few extra static vars to the order
     *
     * @return array
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 26.08.2011
     */
    public function extraStatics() {
        return array(
            'db' => array(
                'InvoiceNumber' => 'VarChar(128)',
            ),
            'has_one' => array(
                'SilvercartPdfInvoiceFile' => 'SilvercartPdfInvoiceFile',
            ),
        );
    }
    
    /**
     * Add a InvoiceNumber to the order
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 26.08.2011
     */
    public function onBeforeWrite() {
        parent::onBeforeWrite();
        if (empty ($this->owner->InvoiceNumber)) {
            $this->owner->InvoiceNumber = SilvercartNumberRange::useReservedNumberByIdentifier('InvoiceNumber');
        }
    }
    
    /**
     * Creates an PDF invoice an attaches it to the email.
     * 
     * @param array &$params Confirmation mail parameters
     *
     * @global array $global_mimetypes
     * 
     * @return bool 
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 25.08.2011
     */
    public function updateConfirmationMail(&$params) {
        global $global_mimetypes;
        
        $pdfInvoice = new SilvercartPdfInvoice($this->owner);
        $this->pdfInvoiceFilePath = $pdfInvoice->generateInvoice();
        // destroy the object to reset the system locale (to prevent ugly price formatting)
        unset($pdfInvoice);
        
        if (!is_array($global_mimetypes)) {
            $global_mimetypes = array();
        }
        if (!array_key_exists('pdf', $global_mimetypes)) {
            $global_mimetypes['pdf'] = 'application/pdf';
        }

        $params['MailOrderConfirmation']['Attachments'] = array(
            $this->pdfInvoiceFilePath,
        );
        $params['MailOrderNotification']['Attachments'] = array(
            $this->pdfInvoiceFilePath,
        );
    }
    
    /**
     * Deletes the PDF invoice if required
     * 
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 26.08.2011
     */
    public function onAfterConfirmationMail() {
        $config = SilvercartPdfInvoiceConfig::getConfig();
        if ($config->SecurityDeleteAfterSending) {
            if (!is_null($this->pdfInvoiceFilePath) &&
                file_exists($this->pdfInvoiceFilePath)) {
                unlink($this->pdfInvoiceFilePath);
            }
        }
    }
    
}