<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Backend
 */

/**
 * The Silvercart configuration backend.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Backend
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @copyright 2011 pixeltricks GmbH
 * @since 02.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartShipmentDhlAdmin extends ModelAdmin {

    /**
     * The code of the menu under which this admin should be shown.
     * 
     * @var string
     */
    public static $menuCode = 'config';

    /**
     * The section of the menu under which this admin should be grouped.
     * 
     * @var string
     */
    public static $menuSortIndex = 90;

    /**
     * The section of the menu under which this admin should be grouped.
     * 
     * @var string
     */
    public static $menuSection = 'shipping';
    
    /**
     * Managed models
     *
     * @var array
     */
    public static $managed_models = array(
        'SilvercartOrder' => array(
            'title'                 => 'SilvercartShipmentDhlExport',
            'collection_controller' => 'SilvercartOrder_CollectionController'
        ),
        'SilvercartShipmentDhlImport' => array(
            'collection_controller' => 'SilvercartShipmentDhlImport_CollectionController'
        ),
        'SilvercartShipmentDhl' => array(
            'record_controller'     => 'SilvercartShipmentDhlAdmin_RecordController',
            'collection_controller' => 'SilvercartShipmentDhl_CollectionController'
        )
    );
    
    /**
     * Defines the importers for the managed models.
     *
     * @var array
     */
    public static $model_importers = array(
        'SilvercartShipmentDhlImport' => 'SilvercartShipmentDhlImportCsvBulkLoader'
    );
    
    /**
     * The URL segment
     *
     * @var string
     */
    public static $url_segment = 'silvercart-shipment-dhl';
    
    /**
     * The menu title
     *
     * @var string
     */
    public static $menu_title = 'Shipment DHL';
    
    /**
     * constructor
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 10.07.2012
     */
    public function __construct() {
        self::$menu_title = _t('SilvercartShipmentDhlAdmin.TITLE');
        
        self::$managed_models['SilvercartOrder']['title']               = _t('SilvercartShipmentDhlOrder.SINGULARNAME');
        
        parent::__construct();
    }
    
    /**
     * We use dropdown menus as selector for this model admin.
     *
     * @return String
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.08.2011
     */
    public function SearchClassSelector() {
        return 'dropdown';
    }
    
    /**
     * Provides a way to use different result tables for the managed models.
     * 
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @copyright 2011 pixeltricks GmbH
     * @since 03.08.2011
     */
    public function resultsTableClassName() {
        $className = $this->resultsTableClassName;
        
        if (isset($this->urlParams['Action']) ) {
            if ($this->urlParams['Action'] == 'SilvercartOrder') {
                $className = 'SilvercartShipmentDhlOrderTableListField';
            }
        }
        
        return $className;
    }
}

/**
 * The Silvercart shipment DHL record controller.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Backend
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @copyright 2011 pixeltricks GmbH
 * @since 02.08.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartShipmentDhlAdmin_RecordController extends ModelAdmin_RecordController {

    /**
     * Adds the ability to execute additional actions to the model admin's
     * action handling.
     *
     * @param SS_HTTPRequest $request The HTTP request
     * 
     * @return mixed
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.08.2011
     */
    public function handleAction(SS_HTTPRequest $request) {
        $vars = $request->requestVars();

        if (array_key_exists('doAttributeItems', $vars)) {
            return $this->doAttributeItems($vars, $request);
        } elseif (array_key_exists('doRemoveItems', $vars)) {
            return $this->doRemoveItems($vars, $request);
        } elseif (array_key_exists('doMoveUpItems', $vars)) {
            return $this->doMoveUpItems($vars, $request);
        } elseif (array_key_exists('doMoveDownItems', $vars)) {
            return $this->doMoveDownItems($vars, $request);
        } elseif (array_key_exists('doAddCallbackField', $vars)) {
            return $this->doAddCallbackField($vars, $request);
        } else {
            return parent::handleAction($request);
        }
    }

    /**
     * We save the CSV-Header definitions here.
     *
     * @param array          $data    The sent data
     * @param Form           $form    The current form object
     * @param SS_HTTPRequest $request The HTTP request
     * 
     * @return mixed
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.08.2011
     */
    public function doSave($data, $form, $request) {

        $shipmentDhlObj = DataObject::get_by_id(
            'SilvercartShipmentDhl',
            Convert::raw2sql($data['ID'])
        );

        if ($shipmentDhlObj) {
            foreach ($data as $httpFieldName => $fieldValue) {
                
                if (substr($httpFieldName, 0, 41) == 'SilvercartShipmentDhlFields_defaultValue_') {
                    $fieldName = substr($httpFieldName, 41);

                    $exporterFieldObj = DataObject::get_one(
                        'SilvercartShipmentDhlField',
                        sprintf(
                            "SilvercartShipmentDhlID = %d AND name = '%s'",
                            $shipmentDhlObj->ID,
                            $fieldName
                        )
                    );

                    if ($exporterFieldObj) {
                        $exporterFieldObj->setField('defaultValue', Convert::raw2sql($fieldValue));
                        $exporterFieldObj->write();
                    }
                }
                
                if (substr($httpFieldName, 0, 32) == 'SilvercartShipmentDhlFields_') {
                    $fieldName = substr($httpFieldName, 32);

                    $exporterFieldObj = DataObject::get_one(
                        'SilvercartShipmentDhlField',
                        sprintf(
                            "SilvercartShipmentDhlID = %d AND name = '%s'",
                            $shipmentDhlObj->ID,
                            $fieldName
                        )
                    );

                    if ($exporterFieldObj) {
                        $exporterFieldObj->setField('headerTitle', Convert::raw2sql($fieldValue));
                        $exporterFieldObj->write();
                    }
                }
            }
        }

        return parent::doSave($data, $form, $request);
    }

    /**
     * Attributes a callback field to the exporter.
     * 
     * @param array          $vars    The request parameters as associative array
     * @param SS_HTTPRequest $request The HTTP request
     * 
     * @return html
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.08.2011
     */
    public function doAddCallbackField($vars, $request) {
        if (isset($vars['ID'])) {
            $shipmentDhlObj = DataObject::get_by_id(
                'SilvercartShipmentDhl',
                Convert::raw2sql($vars['ID'])
            );

            if ($shipmentDhlObj) {
                if (isset($vars['callbackField']) &&
                    !empty($vars['callbackField'])) {

                    $silvercartShipmentDhlField = new SilvercartShipmentDhlField();
                    $silvercartShipmentDhlField->setField('name', Convert::raw2sql($vars['callbackField']));
                    $silvercartShipmentDhlField->setField('SilvercartShipmentDhlID', $shipmentDhlObj->ID);
                    $silvercartShipmentDhlField->setField('sortOrder', $shipmentDhlObj->SilvercartShipmentDhlFields()->Count() + 1);
                    $silvercartShipmentDhlField->setField('isCallbackField', true);
                    $silvercartShipmentDhlField->write();

                    $shipmentDhlObj->SilvercartShipmentDhlFields()->push($silvercartShipmentDhlField);
                    $shipmentDhlObj->write();
                }
            }
        }

        // Behaviour switched on ajax.
        if (Director::is_ajax()) {
                return $this->edit($request);
        } else {
                Director::redirectBack();
        }
    }

    /**
     * Attributes fields to the exporter.
     * 
     * @param array          $vars    The request parameters as associative array
     * @param SS_HTTPRequest $request The HTTP request
     * 
     * @return html
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.08.2011
     */
    public function doAttributeItems($vars, $request) {

        if (isset($vars['ID'])) {
            $shipmentDhlObj = DataObject::get_by_id(
                'SilvercartShipmentDhl',
                Convert::raw2sql($vars['ID'])
            );

            if ($shipmentDhlObj) {
                if (is_array($vars['availableItems'])) {
                    foreach ($vars['availableItems'] as $field) {

                        if (!$shipmentDhlObj->SilvercartShipmentDhlFields()->find('name', $field)) {
                            $silvercartShipmentDhlField = new SilvercartShipmentDhlField();
                            $silvercartShipmentDhlField->setField('name', $field);
                            $silvercartShipmentDhlField->setField('SilvercartShipmentDhlID', $shipmentDhlObj->ID);
                            $silvercartShipmentDhlField->setField('sortOrder', $shipmentDhlObj->SilvercartShipmentDhlFields()->Count() + 1);
                            $silvercartShipmentDhlField->write();

                            $shipmentDhlObj->SilvercartShipmentDhlFields()->push($silvercartShipmentDhlField);
                            $shipmentDhlObj->write();
                        }

                    }
                }
            }
        }

        // Behaviour switched on ajax.
        if (Director::is_ajax()) {
                return $this->edit($request);
        } else {
                Director::redirectBack();
        }
    }

    /**
     * Removes fields from the exporter.
     * 
     * @param array          $vars    The request parameters as associative array
     * @param SS_HTTPRequest $request The HTTP request
     * 
     * @return html
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.08.2011
     */
    public function doRemoveItems($vars, $request) {

        if (isset($vars['ID'])) {
            $shipmentDhlObj = DataObject::get_by_id(
                'SilvercartShipmentDhl',
                Convert::raw2sql($vars['ID'])
            );

            if ($shipmentDhlObj) {
                if (is_array($vars['selectedItems'])) {
                    foreach ($vars['selectedItems'] as $field) {

                        $itemToDelete = $shipmentDhlObj->SilvercartShipmentDhlFields()->find('name', $field);

                        if ($itemToDelete) {
                            $removedSortOrder = $itemToDelete->sortOrder;

                            foreach ($shipmentDhlObj->SilvercartShipmentDhlFields() as $shipmentDhlField) {
                                if ($shipmentDhlField->sortOrder > $removedSortOrder) {
                                    $shipmentDhlField->setField('sortOrder', $shipmentDhlField->sortOrder - 1);
                                    $shipmentDhlField->write();
                                }
                            }

                            $itemToDelete->delete();
                        }
                    }
                }
            }
        }

        // Behaviour switched on ajax.
        if (Director::is_ajax()) {
                return $this->edit($request);
        } else {
                Director::redirectBack();
        }
    }

    /**
     * Moves a field up in the sort hierarchy.
     * 
     * @param array          $vars    The request parameters as associative array
     * @param SS_HTTPRequest $request The HTTP request
     * 
     * @return html
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.08.2011
     */
    public function doMoveUpItems($vars, $request) {

        $itemsToMove = array();

        if (isset($vars['ID'])) {
            $shipmentDhlObj = DataObject::get_by_id(
                'SilvercartShipmentDhl',
                Convert::raw2sql($vars['ID'])
            );

            if ($shipmentDhlObj) {
                if (is_array($vars['selectedItems'])) {
                    foreach ($vars['selectedItems'] as $field) {

                        $itemToMove = $shipmentDhlObj->SilvercartShipmentDhlFields()->find('name', $field);

                        if ($itemToMove) {
                            if ($itemToMove->sortOrder <= $shipmentDhlObj->SilvercartShipmentDhlFields()->Count()) {
                                $itemsToMove['sort_'.str_pad($itemToMove->sortOrder, 10, '0', STR_PAD_LEFT)] = $itemToMove;
                            }
                        }
                    }
                }

                ksort($itemsToMove);

                foreach ($itemsToMove as $sortOrder => $itemToMove) {
                    $moveFromPosition = $itemToMove->sortOrder;
                    $moveToPosition   = $moveFromPosition - 1;

                    $this->changePositions($shipmentDhlObj, $itemToMove, $moveFromPosition, $moveToPosition);
                }
            }
        }

        // Behaviour switched on ajax.
        if (Director::is_ajax()) {
            return $this->edit($request);
        } else {
            Director::redirectBack();
        }
    }

    /**
     * Moves a field down in the sort hierarchy.
     * 
     * @param array          $vars    The request parameters as associative array
     * @param SS_HTTPRequest $request The HTTP request
     * 
     * @return html
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.08.2011
     */
    public function doMoveDownItems($vars, $request) {

        $itemsToMove = array();

        if (isset($vars['ID'])) {
            $shipmentDhlObj = DataObject::get_by_id(
                'SilvercartShipmentDhl',
                Convert::raw2sql($vars['ID'])
            );

            if ($shipmentDhlObj) {
                if (is_array($vars['selectedItems'])) {
                    foreach ($vars['selectedItems'] as $field) {

                        $itemToMove = $shipmentDhlObj->SilvercartShipmentDhlFields()->find('name', $field);

                        if ($itemToMove) {
                            if ($itemToMove->sortOrder < $shipmentDhlObj->SilvercartShipmentDhlFields()->Count()) {
                                $itemsToMove['sort_'.str_pad($itemToMove->sortOrder, 10, '0', STR_PAD_LEFT)] = $itemToMove;
                            }
                        }
                    }
                }

                krsort($itemsToMove);

                foreach ($itemsToMove as $sortOrder => $itemToMove) {
                    $moveFromPosition = $itemToMove->sortOrder;
                    $moveToPosition   = $moveFromPosition + 1;

                    $this->changePositions($shipmentDhlObj, $itemToMove, $moveFromPosition, $moveToPosition);
                }
            }
        }

        // Behaviour switched on ajax.
        if (Director::is_ajax()) {
            return $this->edit($request);
        } else {
            Director::redirectBack();
        }
    }

    /**
     * Changes positions of two SilvercartShipmentDhlFields.
     * 
     * @param SilvercartShipmentDhl      $shipmentDhlObj   The exporter object to which the fields are connected
     * @param SilvercartShipmentDhlField $itemToMove       The item to move
     * @param int                        $moveFromPosition Position to move the field from
     * @param int                        $moveToPosition   Position to move the field to
     * 
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 02.08.2011
     */
    protected function changePositions(SilvercartShipmentDhl $shipmentDhlObj, SilvercartShipmentDhlField $itemToMove, $moveFromPosition, $moveToPosition) {

        if (!$shipmentDhlObj ||
            !$itemToMove) {

            return false;
        }

        $changerObject = DataObject::get_one(
            'SilvercartShipmentDhlField',
            sprintf(
                "SilvercartShipmentDhlID = %d AND sortOrder = %d",
                $shipmentDhlObj->ID,
                $moveToPosition
            )
        );

        if ($changerObject) {
            $changerObject->setField('sortOrder', $moveFromPosition);
            $changerObject->write();

            $itemToMove->setField('sortOrder', $moveToPosition);
            $itemToMove->write();
        }
    }
}
