<?php
/**
 * Copyright 2013 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartShipmentDhl
 * @subpackage Orders
 */

/**
 * Delivers additional information for customers orders.
 *
 * @package SilvercartShipmentDhl
 * @subpacke Orders
 * @author Sascha Koehler <skoehler@pixeltricks.de>,
 *         Sebastian Diel <sdiel@pixeltricks.de>
 * @since 21.11.2013
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartShipmentDhlOrderPlugin extends DataObjectDecorator {
    
    /**
     * Returns a string of HTML code with a link to the track and trace page
     * of DHL.
     * 
     * @param array      &$arguments    Argument list
     * @param DataObject $callingObject Calling object
     *
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>,
     *         Sebastian Diel <sdiel@pixeltricks.de>
     * @since 21.11.2013
     */
    public function pluginOrderDetailInformation(&$arguments, $callingObject = null) {
        $information      = '';
        $showTrackingLink = true;
        if (is_null($callingObject)) {
            $callingObject = $this->owner->getCallingObject();
        }
        
        if (!empty($callingObject->trackingCodeNVE) &&
            !(Controller::curr() instanceof SilvercartOrderDetailPage_Controller &&
              $callingObject->hasMethod('getTrackingCode'))) {
            $shipmentDhl = DataObject::get_one('SilvercartShipmentDhl');

            if ($shipmentDhl) {
                $templateData = array(
                    'SilvercartShipmentDhlTrackingLink' => $showTrackingLink,
                    'Link'                              => $shipmentDhl->trackingUrl . $callingObject->trackingCodeNVE
                );

                $renderer = new ViewableData();

                $information = $renderer->customise($templateData)->renderWith(
                    'SilvercartShipmentDhlTrackingLink'
                );
            }
        }
        return $information;
    }
}